@@ -45,21 +45,12 @@ class EventException(Exception):
     """Exception which is associated with an Event."""
 
     def __init__(self, msg, event):
-        self._event = event
-
-    def getEvent(self):
-        return self._event
-
-    def setEvent(self, event):
-        self._event = event
-
-    event = property(getEvent, setEvent, None, "event property")
-
+        self.args = msg, event
 
 class TaskBase(event.Event):
     """Base class for task events"""
 
-    def __init__(self, t, d = {}):
+    def __init__(self, t, d ):
         self._task = t
         event.Event.__init__(self, d)
 
@@ -123,7 +114,9 @@ def exec_func(func, d, dirs = None):
         exec_func_python(func, d)
     else:
         exec_func_shell(func, d)
-    os.chdir(prevdir)
+
+    if os.path.exists(prevdir):
+        os.chdir(prevdir)
 
 def exec_func_python(func, d):
     """Execute a python BB 'function'"""
@@ -245,11 +238,9 @@ def exec_func_shell(func, d):
             f.close()
         else:
             error("see log in %s" % logfile)
-        raise FuncFailed()
+        raise FuncFailed( logfile )
 
 
-_task_cache = []
-
 def exec_task(task, d):
     """Execute an BB 'task'
 
@@ -267,7 +258,7 @@ def exec_task(task, d):
         task_cache = []
         data.setVar('_task_cache', task_cache, d)
     if not task_graph.hasnode(task):
-        raise EventException("", InvalidTask(task, d))
+        raise EventException("Missing node in task graph", InvalidTask(task, d))
 
     # check whether this task needs executing..
     if not data.getVarFlag(task, 'force', d):
@@ -295,11 +286,12 @@ def exec_task(task, d):
                 exec_func(item, localdata)
                 event.fire(TaskSucceeded(item, localdata))
                 task_cache.append(item)
+                data.setVar('_task_cache', task_cache, d)
             except FuncFailed, reason:
                 note( "Task failed: %s" % reason )
                 failedevent = TaskFailed(item, d)
                 event.fire(failedevent)
-                raise EventException(None, failedevent)
+                raise EventException("Function failed in task: %s" % reason, failedevent)
 
     # execute
     task_graph.walkdown(task, execute)
@@ -364,13 +356,14 @@ def add_task(task, deps, d):
     task_graph = data.getVar('_task_graph', d)
     if not task_graph:
         task_graph = bb.digraph()
-        data.setVar('_task_graph', task_graph, d)
     data.setVarFlag(task, 'task', 1, d)
     task_graph.addnode(task, None)
     for dep in deps:
         if not task_graph.hasnode(dep):
             task_graph.addnode(dep, None)
         task_graph.addnode(task, dep)
+    # don't assume holding a reference
+    data.setVar('_task_graph', task_graph, d)
 
 
 def remove_task(task, kill, d):
@@ -381,7 +374,6 @@ def remove_task(task, kill, d):
     task_graph = data.getVar('_task_graph', d)
     if not task_graph:
         task_graph = bb.digraph()
-        data.setVar('_task_graph', task_graph, d)
     if not task_graph.hasnode(task):
         return
 
@@ -390,6 +382,7 @@ def remove_task(task, kill, d):
     if kill == 1:
         ref = 2
     task_graph.delnode(task, ref)
+    data.setVar('_task_graph', task_graph, d)
 
 def task_exists(task, d):
     task_graph = data.getVar('_task_graph', d)
