@@ -33,15 +33,90 @@ else:
     path = os.path.dirname(os.path.dirname(sys.argv[0]))
 sys.path.append(path)
 
-from bb import note, debug, data_dict
+from bb import note, debug, data_smart
+
+_dict_type = data_smart.DataSmart
+_dict_p_type = data_smart.DataSmartPackage
+
+class DataDictFull(dict):
+    """
+    This implements our Package Data Storage Interface.
+    setDirty is a no op as all items are held in memory
+    """
+    def setDirty(self, bbfile, data):
+        """
+        No-Op we assume data was manipulated as some sort of
+        reference
+        """
+        if not bbfile in self:
+            raise Exception("File %s was not in dictionary before" % bbfile)
+
+        self[bbfile] = data
+
+class DataDictCache:
+    """
+    Databacked Dictionary implementation
+    """
+    def __init__(self, cache_dir):
+        self.cache_dir = cache_dir
+        self.files     = []
+        self.dirty     = {}
+
+    def has_key(self,key):
+        return key in self.files
+
+    def keys(self):
+        return self.files
+
+    def __setitem__(self, key, data):
+        """
+        Add the key to the list of known files and
+        place the data in the cache?
+        """
+        if key in self.files:
+            return
+
+        self.files.append(key)
+
+    def __getitem__(self, key):
+        if not key in self.files:
+            return None
+
+        # if it was dirty we will
+        if key in self.dirty:
+            return self.dirty[key]
+
+        # not cached yet
+        return _dict_p_type(self.cache_dir, key,False,None)
+
+    def setDirty(self, bbfile, data):
+        """
+        Only already added items can be declared dirty!!!
+        """
+
+        if not bbfile in self.files:
+            raise Exception("File %s was not in dictionary before" % bbfile)
+
+        self.dirty[bbfile] = data
 
-_dict_type = data_dict.DataDict
 
 
 def init():
     return _dict_type()
 
-_data_dict = init()
+def init_db(cache,name,clean,parent = None):
+    return _dict_p_type(cache,name,clean,parent)
+
+def init_db_mtime(cache,cache_bbfile):
+    return _dict_p_type.mtime(cache,cache_bbfile)
+
+def pkgdata(use_cache, cache):
+    """
+    Return some sort of dictionary to lookup parsed dictionaires
+    """
+    if use_cache:
+        return DataDictCache(cache)
+    return DataDictFull()
 
 def createCopy(source):
      """Link the source set to the destination
@@ -53,102 +128,110 @@ def createCopy(source):
      """
      return source.createCopy()
 
-def initVar(var, d = _data_dict):
+def initVar(var, d):
     """Non-destructive var init for data structure"""
     d.initVar(var)
 
 
-def setVar(var, value, d = _data_dict):
+def setVar(var, value, d):
     """Set a variable to a given value
 
     Example:
-        >>> setVar('TEST', 'testcontents')
-        >>> print getVar('TEST')
+        >>> d = init()
+        >>> setVar('TEST', 'testcontents', d)
+        >>> print getVar('TEST', d)
         testcontents
     """
     d.setVar(var,value)
 
 
-def getVar(var, d = _data_dict, exp = 0):
+def getVar(var, d, exp = 0):
     """Gets the value of a variable
 
     Example:
-        >>> setVar('TEST', 'testcontents')
-        >>> print getVar('TEST')
+        >>> d = init()
+        >>> setVar('TEST', 'testcontents', d)
+        >>> print getVar('TEST', d)
         testcontents
     """
     return d.getVar(var,exp)
 
-def delVar(var, d = _data_dict):
+def delVar(var, d):
     """Removes a variable from the data set
 
     Example:
-        >>> setVar('TEST', 'testcontents')
-        >>> print getVar('TEST')
+        >>> d = init()
+        >>> setVar('TEST', 'testcontents', d)
+        >>> print getVar('TEST', d)
         testcontents
-        >>> delVar('TEST')
-        >>> print getVar('TEST')
+        >>> delVar('TEST', d)
+        >>> print getVar('TEST', d)
         None
     """
     d.delVar(var)
 
-def setVarFlag(var, flag, flagvalue, d = _data_dict):
+def setVarFlag(var, flag, flagvalue, d):
     """Set a flag for a given variable to a given value
 
     Example:
-        >>> setVarFlag('TEST', 'python', 1)
-        >>> print getVarFlag('TEST', 'python')
+        >>> d = init()
+        >>> setVarFlag('TEST', 'python', 1, d)
+        >>> print getVarFlag('TEST', 'python', d)
         1
     """
     d.setVarFlag(var,flag,flagvalue)
 
-def getVarFlag(var, flag, d = _data_dict):
+def getVarFlag(var, flag, d):
     """Gets given flag from given var
 
     Example:
-        >>> setVarFlag('TEST', 'python', 1)
-        >>> print getVarFlag('TEST', 'python')
+        >>> d = init()
+        >>> setVarFlag('TEST', 'python', 1, d)
+        >>> print getVarFlag('TEST', 'python', d)
         1
     """
     return d.getVarFlag(var,flag)
 
-def delVarFlag(var, flag, d = _data_dict):
+def delVarFlag(var, flag, d):
     """Removes a given flag from the variable's flags
 
     Example:
-        >>> setVarFlag('TEST', 'testflag', 1)
-        >>> print getVarFlag('TEST', 'testflag')
+        >>> d = init()
+        >>> setVarFlag('TEST', 'testflag', 1, d)
+        >>> print getVarFlag('TEST', 'testflag', d)
         1
-        >>> delVarFlag('TEST', 'testflag')
-        >>> print getVarFlag('TEST', 'testflag')
+        >>> delVarFlag('TEST', 'testflag', d)
+        >>> print getVarFlag('TEST', 'testflag', d)
         None
 
     """
     d.delVarFlag(var,flag)
 
-def setVarFlags(var, flags, d = _data_dict):
+def setVarFlags(var, flags, d):
     """Set the flags for a given variable
 
     Example:
+        >>> d = init()
         >>> myflags = {}
         >>> myflags['test'] = 'blah'
-        >>> setVarFlags('TEST', myflags)
-        >>> print getVarFlag('TEST', 'test')
+        >>> setVarFlags('TEST', myflags, d)
+        >>> print getVarFlag('TEST', 'test', d)
         blah
     """
     d.setVarFlags(var,flags)
 
-def getVarFlags(var, d = _data_dict):
+def getVarFlags(var, d):
     """Gets a variable's flags
 
     Example:
-        >>> setVarFlag('TEST', 'test', 'blah')
-        >>> print getVarFlags('TEST')['test']
+        >>> d = init()
+        >>> setVarFlag('TEST', 'test', 'blah', d)
+        >>> print getVarFlags('TEST', d)['test']
         blah
     """
     return d.getVarFlags(var)
 
-def delVarFlags(var, d = _data_dict):
+def delVarFlags(var, d):
     """Removes a variable's flags
 
     Example:
@@ -163,7 +246,7 @@ def delVarFlags(var, d = _data_dict):
     """
     d.delVarFlags(var)
 
-def keys(d = _data_dict):
+def keys(d):
     """Return a list of keys in d
 
     Example:
@@ -176,29 +259,43 @@ def keys(d = _data_dict):
     """
     return d.keys()
 
-def getData(d = _data_dict):
+def getData(d):
     """Returns the data object used"""
     return d
 
-def setData(newData, d = _data_dict):
+def setData(newData, d):
     """Sets the data object to the supplied value"""
     d = newData
 
 __expand_var_regexp__ = re.compile(r"\${[^{}]+}")
 __expand_python_regexp__ = re.compile(r"\${@.+?}")
 
-def expand(s, d = _data_dict, varname = None):
+def expand(s, d, varname = None):
     """Variable expansion using the data store.
 
     Example:
         Standard expansion:
-        >>> setVar('A', 'sshd')
-        >>> print expand('/usr/bin/${A}')
+        >>> d = init()
+        >>> setVar('A', 'sshd', d)
+        >>> print expand('/usr/bin/${A}', d)
         /usr/bin/sshd
 
         Python expansion:
-        >>> print expand('result: ${@37 * 72}')
+        >>> d = init()
+        >>> print expand('result: ${@37 * 72}', d)
         result: 2664
+
+        Shell expansion:
+        >>> d = init()
+        >>> print expand('${TARGET_MOO}', d)
+        ${TARGET_MOO}
+        >>> setVar('TARGET_MOO', 'yupp', d)
+        >>> print expand('${TARGET_MOO}',d)
+        yupp
+        >>> setVar('SRC_URI', 'http://somebug.${TARGET_MOO}', d)
+        >>> delVar('TARGET_MOO', d)
+        >>> print expand('${SRC_URI}', d)
+        http://somebug.${TARGET_MOO}
     """
     def var_sub(match):
         key = match.group()[2:-1]
@@ -238,7 +335,7 @@ def expand(s, d = _data_dict, varname = None):
             raise
     return s
 
-def expandKeys(alterdata = _data_dict, readdata = None):
+def expandKeys(alterdata, readdata = None):
     if readdata == None:
         readdata = alterdata
 
@@ -261,7 +358,7 @@ def expandKeys(alterdata = _data_dict, readdata = None):
 
         delVar(key, alterdata)
 
-def expandData(alterdata = _data_dict, readdata = None):
+def expandData(alterdata, readdata = None):
     """For each variable in alterdata, expand it, and update the var contents.
        Replacements use data from readdata.
 
@@ -288,7 +385,7 @@ def expandData(alterdata = _data_dict, readdata = None):
 
 import os
 
-def inheritFromOS(d = _data_dict):
+def inheritFromOS(d):
     """Inherit variables from the environment."""
 #   fakeroot needs to be able to set these
     non_inherit_vars = [ "LD_LIBRARY_PATH", "LD_PRELOAD" ]
@@ -302,7 +399,7 @@ def inheritFromOS(d = _data_dict):
 
 import sys
 
-def emit_var(var, o=sys.__stdout__, d = _data_dict, all=False):
+def emit_var(var, o=sys.__stdout__, d = init(), all=False):
     """Emit a variable to be sourced by a shell."""
     if getVarFlag(var, "python", d):
         return 0
@@ -352,7 +449,7 @@ def emit_var(var, o=sys.__stdout__, d = _data_dict, all=False):
     return 1
 
 
-def emit_env(o=sys.__stdout__, d = _data_dict, all=False):
+def emit_env(o=sys.__stdout__, d = init(), all=False):
     """Emits all items in the data store in a format such that it can be sourced by a shell."""
 
     env = keys(d)
@@ -367,40 +464,41 @@ def emit_env(o=sys.__stdout__, d = _data_dict, all=False):
             continue
         emit_var(e, o, d) and o.write('\n')
 
-def update_data(d = _data_dict):
+def update_data(d):
     """Modifies the environment vars according to local overrides and commands.
     Examples:
         Appending to a variable:
-        >>> setVar('TEST', 'this is a')
-        >>> setVar('TEST_append', ' test')
-        >>> setVar('TEST_append', ' of the emergency broadcast system.')
-        >>> update_data()
-        >>> print getVar('TEST')
+        >>> d = init()
+        >>> setVar('TEST', 'this is a', d)
+        >>> setVar('TEST_append', ' test', d)
+        >>> setVar('TEST_append', ' of the emergency broadcast system.', d)
+        >>> update_data(d)
+        >>> print getVar('TEST', d)
         this is a test of the emergency broadcast system.
 
         Prepending to a variable:
-        >>> setVar('TEST', 'virtual/libc')
-        >>> setVar('TEST_prepend', 'virtual/tmake ')
-        >>> setVar('TEST_prepend', 'virtual/patcher ')
-        >>> update_data()
-        >>> print getVar('TEST')
+        >>> setVar('TEST', 'virtual/libc', d)
+        >>> setVar('TEST_prepend', 'virtual/tmake ', d)
+        >>> setVar('TEST_prepend', 'virtual/patcher ', d)
+        >>> update_data(d)
+        >>> print getVar('TEST', d)
         virtual/patcher virtual/tmake virtual/libc
 
         Overrides:
-        >>> setVar('TEST_arm', 'target')
-        >>> setVar('TEST_ramses', 'machine')
-        >>> setVar('TEST_local', 'local')
-        >>> setVar('OVERRIDES', 'arm')
+        >>> setVar('TEST_arm', 'target', d)
+        >>> setVar('TEST_ramses', 'machine', d)
+        >>> setVar('TEST_local', 'local', d)
+        >>> setVar('OVERRIDES', 'arm', d)
 
-        >>> setVar('TEST', 'original')
-        >>> update_data()
-        >>> print getVar('TEST')
+        >>> setVar('TEST', 'original', d)
+        >>> update_data(d)
+        >>> print getVar('TEST', d)
         target
 
-        >>> setVar('OVERRIDES', 'arm:ramses:local')
-        >>> setVar('TEST', 'original')
-        >>> update_data()
-        >>> print getVar('TEST')
+        >>> setVar('OVERRIDES', 'arm:ramses:local', d)
+        >>> setVar('TEST', 'original', d)
+        >>> update_data(d)
+        >>> print getVar('TEST', d)
         local
     """
 
@@ -410,7 +508,7 @@ def update_data(d = _data_dict):
     dodel = []
     overrides = (getVar('OVERRIDES', d, 1) or "").split(':') or []
 
-    def applyOverrides(var, d = _data_dict):
+    def applyOverrides(var, d):
         if not overrides:
             debug(1, "OVERRIDES not defined, nothing to do")
             return
